with Execution_Load;
with Calendar; use Calendar;
with Priority_Queue;
with Event_Monitor; use Event_Monitor;
with Task_Params; use Task_Params;
with Ada.Numerics.Float_Random; use Ada.Numerics;
with Ada.Exceptions; use Ada;

package body Dispatcher is

   task body Dispatch is

      Params : array (Task_Id) of Task_Parameters;

      Initial_Time   : CALENDAR.TIME;
      The_Task       : Task_Id;
      Next_Time      : DURATION;

      Gen : Float_Random.Generator;

      package Activation_Queue is new Priority_Queue
        (Size       => 20,
         Element    => Task_Id,
         Priority   => Duration,
         ">"        => "<",
         "="        => "=");

      function Proximo_Evento (La_Tarea : in Task_Id) return Duration is
      begin
         return (Duration(Float(Params(La_Tarea).Avg_Interarrival)*
                          2.0*Float_Random.Random(Gen)));
      end Proximo_Evento;

   begin
      -- Inicializacion
      Float_Random.Reset(Gen);
      for I in Task_Id range 1..Number_Of_Tasks loop
         Params(I):=Get_Parameters(I);
         if Params(I).Event_Kind=Periodic then
            Activation_Queue.Enqueue(I,Params(I).Phase);
         else
            Activation_Queue.Enqueue(I,Proximo_Evento(I));
         end if;
      end loop;
      accept Start;
      Initial_Time:=CLOCK;
      Monitor.Start(Initial_Time);

      -- Lazo Principal

      loop
         Activation_Queue.Dequeue(The_Task,Next_Time);
         delay until (Initial_Time + Next_Time);
         Monitor.Send_Event(The_Task);
         if Params(The_Task).Event_Kind=Periodic then
            Activation_Queue.Enqueue
              (The_Task,Next_Time+Params(The_Task).Period);
         else
            Activation_Queue.Enqueue
              (The_Task,Next_Time+Proximo_Evento(The_Task));
         end if;
      end loop;

   exception

      when Error : others           =>
         Monitor.Put_Message ("exception: " &
                                  Exceptions.Exception_Name(Error));

   end Dispatch;

end Dispatcher;

