with Calendar; use Calendar;
with Queues;
with Task_Params; use type Task_Params.Task_Id;

package body Event_Monitor is

   type Task_Time_Parameters is array (Task_Params.Task_Id) of Duration;

   Deadlines     : Task_Time_Parameters;

   package Start_Times is new Queues
     (Queue_Size => 20,
      Element    => Duration);
   Task_Start : array (Task_Params.Task_Id) of Start_Times.Queue;

   package Records is new Queues
     (Queue_Size => 1000,
      Element    => Record_Type);
   Recs : Records.Queue;

   package Messages is new Queues
     (Queue_Size => 1000,
      Element    => Message);
   Msgs : Messages.Queue;


   protected body Monitor is

      procedure Start (Init_Time   : in Calendar.Time) is
      begin
         Initial_Time:=Init_Time;
      end Start;

      procedure Send_Event(The_Task : in Task_Params.Task_Id) is
         Now : Duration:=Clock-Initial_Time;
         Rec : Record_Type;
      begin
         Start_Times.Enqueue(Now,Task_Start(The_Task));
         Current_State(The_Task).Num_Events:=
           Current_State(The_Task).Num_Events+1;
         Rec.Event_Kind:=Send_Event;
         Rec.The_Task:=The_Task;
         Rec.Instant:=Now;
         Rec.State:=Current_State(The_Task);
         Records.Enqueue(Rec,Recs);
      end Send_Event;

      procedure End_Of_Job(The_Task : in Task_Params.Task_Id) is

         Now : Duration:=Clock-Initial_Time;
         Activation_Time : Duration;
         Rec : Record_Type;
      begin
         Current_State(The_Task).Num_Events:=
           Current_State(The_Task).Num_Events-1;
         Current_State(The_Task).Num_Processed:=
           Current_State(The_Task).Num_Processed+1;
         Rec.Event_Kind:=End_Of_Job;
         Rec.The_Task:=The_Task;
         Rec.Instant:=Now;
         Start_Times.Dequeue(Activation_Time,Task_Start(The_Task));
         Rec.Response_Time:=Now-Activation_Time;
         if Rec.Response_Time>Current_State(The_Task).Worst then
            Current_State(The_Task).Worst:=Rec.Response_Time;
         end if;
         Current_State(The_Task).Sum_Of_Resp:=
           Current_State(The_Task).Sum_Of_Resp+Rec.Response_Time;

         if Rec.Response_Time>Deadlines(The_Task) then
            Current_State(The_Task).Missed_Deadlines:=
              Current_State(The_Task).Missed_Deadlines+1;
         end if;
         Rec.State:=Current_State(The_Task);
         Records.Enqueue(Rec,Recs);
      end End_Of_Job;


      entry Get_Record (Rec : out Record_Type)
      when not Records.Empty(Recs) is
      begin
         Records.Dequeue(Rec,Recs);
      end Get_Record;


      procedure Put_Message (Msg : in String) is
         The_Msg : Message:="                              ";
         Now : Duration:=Clock-Initial_Time;
         Rec : Record_Type;
      begin
         if Msg'Length>=The_Msg'Length then
            The_Msg:=Msg
              (Msg'First..Msg'First+The_Msg'Length-1);
         else
            The_Msg(1..Msg'Length):=Msg;
         end if;
         Messages.Enqueue(The_Msg,Msgs);
         Rec.Event_Kind:=Message_Inserted;
         Rec.Instant:=Now;
         Records.Enqueue(Rec,Recs);
      exception
         when Messages.Full_Queue =>
            null;
      end Put_Message;


      procedure Get_Message (Msg : out Message;
                                Available : out Boolean) is
      begin
         Available:=not Messages.Empty(Msgs);
         if Available then
            Messages.Dequeue(Msg,Msgs);
         end if;
      end Get_Message;

      entry Wait_For_Event (for I in Task_Params.Task_Id)
      when Current_State(I).Num_Events>0 is
      begin
         null;
      end Wait_For_Event;

   end Monitor;

begin

   for T in Task_Params.Task_Id range 1..Task_Params.Number_Of_Tasks loop
      Deadlines(T):=Task_Params.Get_Parameters(T).Deadline;
   end loop;

end Event_Monitor;




