with Calendar;
with Task_Params;
package Event_Monitor is

   subtype Message is String (1..30);

   type Event_Kinds is (Send_Event,End_Of_Job,Message_Inserted);

   type Task_State is record
      Num_Events       : Natural:=0;
      Missed_Deadlines : Natural:=0;
      Worst            : Duration:=0.0;
      Sum_Of_Resp      : Duration:= 0.0;
      Num_Processed    : Natural:=0;
   end record;

   type States is array (Task_Params.Task_Id) of Task_State;

   type Record_Type is record
      Event_Kind : Event_Kinds;
      The_Task : Task_Params.Task_Id;
      Instant : Duration;
      Response_Time : Duration:=0.0;
      State  : Task_State;
   end record;

   protected Monitor is
      procedure Start
        (Init_Time : in Calendar.Time);
      procedure Send_Event
        (The_Task : in Task_Params.Task_Id);
      entry Wait_For_Event
        (Task_Params.Task_Id);  -- family of entries
      procedure End_Of_Job
        (The_Task : in Task_Params.Task_Id);
      entry Get_Record
        (Rec : out Record_Type);
      procedure Put_Message
        (Msg  : in String);
      procedure Get_Message
        (Msg : out Message;
         Available : out Boolean);
      pragma Priority(20);
   private
      Current_State : States;
      Initial_Time  : Calendar.Time;
   end Monitor;

end Event_Monitor;
